/**
 ******************************************************************************
 * @file    lis331dlh.c
 * @author  Radek Kysilka
 * @version V1.0.0
 * @date    14-February-2018
 * @brief   This file provides a set of functions needed to manage the LIS2DH12
 *          accelero and gyro devices
 ******************************************************************************
 * @attention
 *
 ******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "lis331dlh.h"

ACCELERO_DrvTypeDef Lis331dlhAccDrv =
{
  LIS331DLH_AccInit,
  LIS331DLH_AccDeInit,
  LIS331DLH_AccReadID,
  0,
  LIS331DLH_AccLowPower,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  LIS331DLH_AccReadXYZ
};


/**
  * @brief  Set LIS331DLH Accelerometer Initialization.
  * @param  InitStruct: Init parameters
  * CTRL_REG1 (20h) ===> MSB  
  * CTRL_REG4 (23h) ===> LSB  
  */
void LIS331DLH_AccInit(uint16_t InitStruct)
{  
  //uint8_t ctrl = 0x00;
  uint8_t tmp;

  /* Write value to ACC MEMS CTRL4 register: FS and Data Rate */
  tmp = (uint8_t) InitStruct;
  SENSOR_IO_Write(BSP_I2C_ADDRESS_ACC_LIS331DLH, LIS331DLH_ACC_CTRL_REG4, tmp);
  tmp = ((uint8_t) (InitStruct >> 8)); 
  SENSOR_IO_Write(BSP_I2C_ADDRESS_ACC_LIS331DLH, LIS331DLH_ACC_CTRL_REG1, tmp);
  //SENSOR_IO_Write(BSP_I2C_ADDRESS_ACC_LIS331DLH, LIS331DLH_ACC_CTRL_REG3, 0x80); /* Set Interrupt as active low push pull */
}

/**
  * @brief  LIS331DLH Accelerometer De-initialization.
  */
void LIS331DLH_AccDeInit(void)
{
  uint8_t ctrl = 0x00;
  
  /* Read control register 1 value */
  ctrl = SENSOR_IO_Read(BSP_I2C_ADDRESS_ACC_LIS331DLH, LIS331DLH_ACC_CTRL_REG1);

  /* Clear ODR bits */
  ctrl &= ~(LIS331DLH_ACC_ODR_BITPOSITION | LIS331DLH_ACC_POWERMODE_BITPOSITION);

  /* Set Power down */
  ctrl |= LIS331DLH_ACC_POWER_DOWN;
  
  /* write back control register */
  SENSOR_IO_Write(BSP_I2C_ADDRESS_ACC_LIS331DLH, LIS331DLH_ACC_CTRL_REG1, ctrl);
}

/**
  * @brief  Read LIS331DLH ID.
  * @retval ID 
  */
uint8_t LIS331DLH_AccReadID(void)
{  
  /* IO interface initialization */
  SENSOR_IO_Init();
  /* Read value at Who am I register address */
  return (SENSOR_IO_Read(BSP_I2C_ADDRESS_ACC_LIS331DLH, LIS331DLH_ACC_WHO_AM_I_REG));
}

/**
  * @brief  Set/Unset Accelerometer in low power mode.
  * @param  status=0  Normal Mode, status=1 Power Down Mode
  */
void LIS331DLH_AccLowPower(uint16_t status)
{
  uint8_t ctrl = 0x00;
  
  /* Read CTRL6_C value */
  ctrl = SENSOR_IO_Read(BSP_I2C_ADDRESS_ACC_LIS331DLH, LIS331DLH_ACC_CTRL_REG1);

  /* Clear Low Power Mode bit */
  ctrl &= ~(LIS331DLH_ACC_POWERMODE_BITPOSITION);

  /* Set Low Power Mode */
  if(status)
  {
    ctrl |= LIS331DLH_ACC_POWER_DOWN;
  }else
  {
    ctrl |= LIS331DLH_ACC_POWER_NORMAL;
  }
  
  /* write back control register */
  SENSOR_IO_Write(BSP_I2C_ADDRESS_ACC_LIS331DLH, LIS331DLH_ACC_CTRL_REG1, ctrl);
}

/**
  * @brief  Read X, Y & Z Acceleration values 
  * @param  pData: Data out pointer
  */
void LIS331DLH_AccReadXYZ(int16_t* pData)
{
  int16_t pnRawData[3];
  uint8_t ctrlx= 0;
  uint8_t buffer[6];
  uint8_t i = 0;
  float sensitivity = 0;
  
  /* Read the acceleration control register content */
  ctrlx = SENSOR_IO_Read(BSP_I2C_ADDRESS_ACC_LIS331DLH, LIS331DLH_ACC_CTRL_REG4);
  
  /* Read output register X, Y & Z acceleration */
  for(i=0; i<6; i++)
  {
	  buffer[i] =  SENSOR_IO_Read(BSP_I2C_ADDRESS_ACC_LIS331DLH, LIS331DLH_ACC_OUT_X_L+i);
  }
  
  for(i=0; i<3; i++)
  {
    pnRawData[i]=((((uint16_t)buffer[2*i+1]) << 8) + (uint16_t)buffer[2*i]);
  }
  
  /* Normal mode */
  /* Switch the sensitivity value set in the CRTL1_XL */
  switch(ctrlx & LIS331DLH_ACC_FULLSCALE_BITPOSITION)
  {
  case LIS331DLH_ACC_FULLSCALE_2G:
    sensitivity = LIS331DLH_ACC_SENSITIVITY_2G;
    break;
  case LIS331DLH_ACC_FULLSCALE_4G:
    sensitivity = LIS331DLH_ACC_SENSITIVITY_4G;
    break;
  case LIS331DLH_ACC_FULLSCALE_8G:
    sensitivity = LIS331DLH_ACC_SENSITIVITY_8G;
    break;
  }
  
  /* Obtain the mg value for the three axis */
  for(i=0; i<3; i++)
  {
    pData[i]=( int16_t )(pnRawData[i] * sensitivity);
  }
}


